io.stdout:setvbuf("no")
W, H = love.graphics.getDimensions()

cpml = require "cpml"

local vmax = 5
local near, far = 0.0625, 1024.0625

local atypes = {
	{"VertexPosition", "float", 3},
	{"VertexTexCoord", "float", 2},
}

local axes = {
	x = cpml.vec3.new(1, 0, 0),
	y = cpml.vec3.new(0, 1, 0),
	z = cpml.vec3.new(0, 0, 1),
}

function love.load()
	camera = cpml.mat4.from_perspective(60, 4/3, 0.0625, 1024.0625)
	premultz = cpml.mat4.from_angle_axis(math.pi/2, axes.z)
	premultx = cpml.mat4.from_angle_axis(math.pi/2, axes.x)
	cpml.mat4.mul(camera, camera, premultz)
	cpml.mat4.mul(camera, camera, premultx)

	print(camera)

    posrot = cpml.mat4.identity()

    player = {
    	pos = cpml.vec3(),
    	rot = 0,
	}

	s3d = love.graphics.newShader(
	[[
		vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords )
		{
			vec4 texcolor = Texel(texture, texture_coords);
			return texcolor * color;
		}
	]],
	[[
		extern mat4 view;
		vec4 position(mat4 transform, vec4 vertex)
		{
			return view * vertex;
		}
	]])

	depthBuffer = love.graphics.newCanvas(W, H, {format="depth16", readable=true})

	img = love.graphics.newImage("floor.png")

	-- A spritebatch is a mesh, so let's use that for simplicity
	sb = love.graphics.newSpriteBatch(img, 1)
	-- Generate 8x8 quads (image is 640x480 which means each quad is 80x60)
	local quads = {}
	quads[0] = love.graphics.newQuad(0, 0, 600, 600, 600, 600)
	-- Add some quads at random
	sb:add(quads[0], 0, 0)

	love.graphics.setShader(s3d)
end

function love.draw()
	local res = cpml.mat4()
	--cpml.mat4.mul(res, posrot, camera)
	cpml.mat4.transpose(res, camera)
	

	if (os.time() % 2 == 0) then
		s3d:send("view", res)
	else
		cpml.mat4.transpose(res, cpml.mat4 {
		     1.3,    0,  0,  0,
		       0,    0,  1,  1,
		       0, -1.7,  0,  0,
		    -450, -120, 50, 50,
	  	})

		s3d:send("view", res)
	end

	love.graphics.draw(sb)
end

function love.update(dt)
	local omega, speed = math.pi/4, 100

	cpml.mat4.translate(camera, camera, -player.pos)
	cpml.mat4.rotate(camera, camera, -player.rot, axes.x)

	if love.keyboard.isDown("left") then
		player.rot = (player.rot + omega * dt) % (2*math.pi)
	end
	if love.keyboard.isDown("right") then
		player.rot = (player.rot - omega * dt) % (2*math.pi)
	end

	local dirVector = cpml.vec3(math.cos(player.rot), math.sin(player.rot), 0)
	if love.keyboard.isDown("up") then
		player.pos = player.pos + dirVector * speed * dt
	end
	if love.keyboard.isDown("down") then
		player.pos = player.pos - dirVector * speed * dt
	end
	
	cpml.mat4.rotate(camera, camera, player.rot, axes.x)
	cpml.mat4.translate(camera, camera, player.pos)
end